#include "AD9951.h"

void init_AD9951()
{
    setPinDirExt(PORT_SDO, PIN_SDO, GPIO_INPUT, GPIO_NO_PULLUP_PULLDOWN);
    setPinDir(PORT_SDIO, PIN_SDIO, GPIO_OUTPUT);
    setPinDir(PORT_SCLK, PIN_SCLK, GPIO_OUTPUT);
    setPinDir(PORT_CS, PIN_CS, GPIO_OUTPUT);
    setPinDir(PORT_RESET, PIN_RESET, GPIO_OUTPUT);
    setPinDir(PORT_IO_UPDATE, PIN_IO_UPDATE, GPIO_OUTPUT);

    setPinState(PORT_RESET, PIN_RESET, 0);
    wait_cycles(1000);
    setPinState(PORT_CS, PIN_CS, 1);
    setPinState(PORT_SCLK, PIN_SCLK, 0);
    setPinState(PORT_SDIO, PIN_SDIO, 0);
    setPinState(PORT_IO_UPDATE, PIN_IO_UPDATE, 0);
}

uint8_t byte_transfer_AD9951(uint8_t byte_in)
{
    setPinState(PORT_SCLK, PIN_SCLK, 0);
    wait_cycles(100);
    setPinState(PORT_CS, PIN_CS, 0);
    wait_cycles(100);

    uint8_t rec_data = 0x00;
    for(uint8_t i = 0; i < 8; i++)
    {
        setPinState(PORT_SDIO, PIN_SDIO, (0x01 & (byte_in >> (7 - i))));
        wait_cycles(100);
        setPinState(PORT_SCLK, PIN_SCLK, 1);
        wait_cycles(100);
        rec_data |= (getPinState(PORT_SDO, PIN_SDO) << (7 - i));
        wait_cycles(100);
        setPinState(PORT_SCLK, PIN_SCLK, 0);
    }

    wait_cycles(100);
    setPinState(PORT_CS, PIN_CS, 1);
    setPinState(PORT_SCLK, PIN_SCLK, 0);
    wait_cycles(100);

    return rec_data;
}

void configure_AD9951(uint32_t freq)
{
    setPinState(PORT_IO_UPDATE, PIN_IO_UPDATE, 0);
    wait_cycles(100);

    byte_transfer_AD9951(0x00);    // CFR1
    byte_transfer_AD9951(0x00);
    byte_transfer_AD9951(0x00);
    byte_transfer_AD9951(0x02);
    byte_transfer_AD9951(0x00);

    byte_transfer_AD9951(0x01);    // CFR2
    byte_transfer_AD9951(0x00);
    byte_transfer_AD9951(0x00);
    byte_transfer_AD9951(0x00);

    byte_transfer_AD9951(0x02);    // ASF
    byte_transfer_AD9951(0x00);
    byte_transfer_AD9951(0x00);

    byte_transfer_AD9951(0x03);    // ARR
    byte_transfer_AD9951(0x00);

    //byte_transfer_AD9951(0x04);    // FTWO 10.7MHz
    //byte_transfer_AD9951(0x15);
    //byte_transfer_AD9951(0xE9);
    //byte_transfer_AD9951(0xE1);
    //byte_transfer_AD9951(0xB1);

    //byte_transfer_AD9951(0x04);    // FTWO 45MHz
    //byte_transfer_AD9951(0x5C);
    //byte_transfer_AD9951(0x28);
    //byte_transfer_AD9951(0xF5);
    //byte_transfer_AD9951(0xC3);

    //byte_transfer_AD9951(0x04);    // FTWO 45.01MHz
    //byte_transfer_AD9951(0x5C);
    //byte_transfer_AD9951(0x2E);
    //byte_transfer_AD9951(0x33);
    //byte_transfer_AD9951(0xF0);

    uint32_t FTW = (uint32_t)(((uint64_t)(0x100000000ul) * (uint64_t)(freq)) / ((uint64_t)(125000ul)));
    //uint32_t FTW = (((1 << 17) * freq) / 125000) * (1 << 15);

    byte_transfer_AD9951(0x04);    // FTWO
    byte_transfer_AD9951((uint8_t)(FTW >> 24));
    byte_transfer_AD9951((uint8_t)(FTW >> 16));
    byte_transfer_AD9951((uint8_t)(FTW >>  8));
    byte_transfer_AD9951((uint8_t)(FTW)      );

    byte_transfer_AD9951(0x05);    // POWO
    byte_transfer_AD9951(0x00);
    byte_transfer_AD9951(0x00);

    wait_cycles(100);
    setPinState(PORT_IO_UPDATE, PIN_IO_UPDATE, 1);
    wait_cycles(1000);
    setPinState(PORT_IO_UPDATE, PIN_IO_UPDATE, 0);
    wait_cycles(100);
}
