#include "LCD.h"

void initLCD()
{
    setPinDir(RS_PORT, RS_PIN, GPIO_OUTPUT);
    setPinDir(RW_PORT, RW_PIN, GPIO_OUTPUT);
    setPinDir(E_PORT, E_PIN, GPIO_OUTPUT);
    setPinDir(D4_PORT, D4_PIN, GPIO_OUTPUT);
    setPinDir(D5_PORT, D5_PIN, GPIO_OUTPUT);
    setPinDir(D6_PORT, D6_PIN, GPIO_OUTPUT);
    setPinDir(D7_PORT, D7_PIN, GPIO_OUTPUT);
    
    setPinState(E_PORT, E_PIN, 0);
    setPinState(RS_PORT, RS_PIN, 0);
    setPinState(RW_PORT, RW_PIN, 0);
    
    wait_cycles(WAIT_LONG);
    write_data_LCD_single(0x30, WAIT_SHORT, 0);    // soft reset_LCD
    wait_cycles(WAIT_LONG);
    write_data_LCD_single(0x30, WAIT_SHORT, 0);    // soft reset_LCD
    wait_cycles(WAIT_LONG);
    write_data_LCD_single(0x30, WAIT_SHORT, 0);    // soft reset_LCD
    wait_cycles(WAIT_LONG);
    
    write_data_LCD_single(0x20, WAIT_SHORT, 0);    // write_data_LCD(0x38, 500, 0);    // set function LCD
    wait_cycles(WAIT_SHORT);
    write_data_LCD(0x28, WAIT_SHORT, 0);    // set function LCD
    wait_cycles(WAIT_SHORT);
    write_data_LCD(0x08, WAIT_SHORT, 0);    // display off LCD
    wait_cycles(WAIT_SHORT);
    write_data_LCD(0x01, WAIT_SHORT, 0);    // display clear
    wait_cycles(WAIT_LONG);
    write_data_LCD(0x06, WAIT_SHORT, 0);    // entry mode LCD
    wait_cycles(WAIT_SHORT);
    write_data_LCD(0x0C, WAIT_SHORT, 0);    // display on LCD
    wait_cycles(WAIT_LONG);
}

void set_data_LCD(uint8_t data, uint8_t nibble)
{
    if(nibble == HIGH_NIBBLE)
    {
        setPinState(D4_PORT, D4_PIN, ((data >> 4) & 0x01));
        setPinState(D5_PORT, D5_PIN, ((data >> 5) & 0x01));
        setPinState(D6_PORT, D6_PIN, ((data >> 6) & 0x01));
        setPinState(D7_PORT, D7_PIN, ((data >> 7) & 0x01));
    }
    else
    {
        setPinState(D4_PORT, D4_PIN, ((data >> 0) & 0x01));
        setPinState(D5_PORT, D5_PIN, ((data >> 1) & 0x01));
        setPinState(D6_PORT, D6_PIN, ((data >> 2) & 0x01));
        setPinState(D7_PORT, D7_PIN, ((data >> 3) & 0x01));
    }
}

void write_data_LCD_single(uint8_t data, uint32_t N_wait, uint8_t RS_state)
{
    wait_cycles(N_wait);
    setPinState(E_PORT, E_PIN, 0);
    setPinState(RS_PORT, RS_PIN, RS_state);
    setPinState(RW_PORT, RW_PIN, 0);
    wait_cycles(N_wait);
    
    set_data_LCD(data, HIGH_NIBBLE);
    
    wait_cycles(N_wait);
    setPinState(E_PORT, E_PIN, 1);
    wait_cycles(N_wait);
    setPinState(E_PORT, E_PIN, 0);
    wait_cycles(N_wait);
    
    setPinState(RS_PORT, RS_PIN, 0);
    wait_cycles(N_wait);
}

void write_data_LCD(uint8_t data, uint32_t N_wait, uint8_t RS_state)
{
    wait_cycles(N_wait);
    setPinState(E_PORT, E_PIN, 0);
    setPinState(RS_PORT, RS_PIN, RS_state);
    setPinState(RW_PORT, RW_PIN, 0);
    wait_cycles(N_wait);
    
    set_data_LCD(data, HIGH_NIBBLE);
    
    wait_cycles(N_wait);
    setPinState(E_PORT, E_PIN, 1);
    wait_cycles(N_wait);
    setPinState(E_PORT, E_PIN, 0);
    wait_cycles(N_wait);
    
    set_data_LCD(data, LOW_NIBBLE);
    
    wait_cycles(N_wait);
    setPinState(E_PORT, E_PIN, 1);
    wait_cycles(N_wait);
    setPinState(E_PORT, E_PIN, 0);
    wait_cycles(N_wait);
    
    setPinState(RS_PORT, RS_PIN, 0);
    wait_cycles(N_wait);
}

void write_string_LCD(char *str)
{
    uint16_t cnt = 0;
    write_data_LCD((0x80 | 0x00), WAIT_SHORT, 0);    // Set DD RAM address
    
    while((*str != '\0') && (cnt <= 32))
    {
        write_data_LCD(*str, WAIT_SHORT, 1);
        str++;
        cnt++;
        
        if(cnt == 16)
        {
            write_data_LCD((0x80 | 0x40), WAIT_SHORT, 0);    // Set DD RAM address
        }
    }
}

void write_uint32_LCD(uint32_t number)
{
    char buff[17];
    uint8_t cnt = 0;
    for(uint8_t i = 0; i < 10; i++)
    {
        buff[9 - cnt] = (number % 10) + 48;
        cnt++;
        number = number / 10;
    }
    
    
    while(cnt < 16)
    {
        buff[cnt] = ' ';
        cnt++;
    }
    
    buff[16] = '\0';
    
    write_string_LCD(buff);
}

void write_freq_locked_LCD(uint32_t freq, uint8_t locked, uint32_t freq_step)
{
    char buff[33];
    
    for(uint8_t i = 0; i < 7; i++)
    {
        buff[6 - i] = (freq % 10) + 48;
        freq = freq / 10;
    }
    
    uint8_t i_r1 = 0;
    while((buff[i_r1] == '0') && (i_r1 < 7))
    {
        buff[i_r1] = ' ';
        i_r1++;
    }
    
    buff[7] = ' ';
    buff[8] = 'k';
    buff[9] = 'H';
    buff[10] = 'z';
    buff[11] = ' ';
    
    if(locked == 1)
    {
        buff[12] = 'l';
        buff[13] = 'o';
        buff[14] = 'c';
        buff[15] = 'k';
    }
    else
    {
        buff[12] = ' ';
        buff[13] = ' ';
        buff[14] = ' ';
        buff[15] = ' ';
    }
    
    for(uint8_t i = 0; i < 5; i++)
    {
        buff[20 - i] = (freq_step % 10) + 48;
        freq_step = freq_step / 10;
    }
    
    uint8_t i_r2 = 16;
    while((buff[i_r2] == '0') && (i_r2 < 21))
    {
        buff[i_r2] = ' ';
        i_r2++;
    }
    
    buff[21] = ' ';
    buff[22] = 'k';
    buff[23] = 'H';
    buff[24] = 'z';
    buff[25] = ' ';
    buff[26] = 's';
    buff[27] = 't';
    buff[28] = 'e';
    buff[29] = 'p';
    buff[30] = 's';
    
    buff[31] = ' ';
    
    buff[32] = '\0';
    
    write_string_LCD(buff);
}
